import std.string;
import std.path;

import config;


class MenuGenerator
{
    this(string[string][string] _infoList, Config _cfg, string[] _argv)
    {
        infoList = _infoList;
        keysList = infoList.keys.sort;
        cfg = _cfg;
        argv = _argv;
        
        initCategoriesInfo();
    }
    
    
    string getStaticMenuXML()
    {
        return getMenu("staticXML");
    }
    
    
    string getPipeMenuXML()
    {
        return getMenu("pipemenuXML");
    }
    
    
    string getPipeMenu()
    {
        return getMenu("pipemenu");
    }
    
    
    
    private:
        string[string][string] infoList;
        Config cfg;
        string[] argv;
        string[] keysList;
        
        struct CategoryInfo { string[] categories; string label; };
        CategoryInfo[string] CategoriesInfo;
        
        string SUBMENU = "    <menu id=\"%s\" label=\"%s\">\n%s    </menu>\n";
        string MENU_ITEM = "        <item label=\"%s\"><action name=\"Execute\"><execute>%s</execute></action></item>\n";

        

        string getMenu(string menu_type)
        {
            string menu;
            
            menu ~= getHeader(menu_type);
            
            if (menu_type != "pipemenuXML")
            {
                menu ~= getTopMenu();
                
                if (cfg.getValue("show windows and desktops menu") == "true")
                    menu ~= getWindowsAndDesktopsMenu();
                
                if (cfg.getValue("show apps section label") == "true")
                    menu ~= separator("Applications");
                else
                    menu ~= separator();
                
                foreach(category; ["accesories", "graphics", "audiovideo", "office", "games", "network", "devel", "settings", "system"])
                    menu ~= getCategoryMenu(category);
                
                menu ~= getOpenBoxMenu();
                menu ~= getLockAndExitItems();
            }
            
            menu ~= getFooter(menu_type);
            
            return menu;
        }


        string getHeader(string menu_type)
        {
            string header = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
            "<openbox_menu xmlns=\"http://openbox.org/\" "
            "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" "
            "xsi:schemaLocation=\"http://openbox.org/ file:///usr/X11R6/share/openbox/menu.xsd\">\n";
            
            if (menu_type == "staticXML")
            {
                header ~= "<menu id=\"root-menu\" label=\"openbox3\">\n";
            }
            else if (menu_type == "pipemenuXML")
            {
                string appPath = argv[0];
                if (startsWith(appPath, ".")) appPath = rel2abs(appPath);
                string command = appPath ~ " --pipe";
                if (argv.length > 2)
                {
                    string extraParams = std.string.join(argv[2..length], "");
                    command ~= " " ~ extraParams;
                }
                header ~= format("<menu id=\"root-menu\" label=\"openbox3\" execute=\"%s\" />\n", command);
            }
            else if (menu_type == "pipemenu")
            {
                header = "<openbox_pipe_menu>\n";
            }
            return header;
        }

        
        string getTopMenu()
        {
            string terminal;
            if (cfg.getValue("terminal") == "none")
                throw new Exception("Configuration key 'terminal' MUST be setted.");
            else if (cfg.getValue("show terminal item") != "false")
                terminal = format(MENU_ITEM[4..length], "Terminal", cfg.getValue("terminal"));
                
            string editor;
            if (cfg.getValue("editor") == "none")
                throw new Exception("Configuration key 'editor' MUST be setted.");
            else if (cfg.getValue("show editor item") != "false")
                editor = format(MENU_ITEM[4..length], "Editor", cfg.getValue("editor"));
                
            string filemanager = "";
            if ((cfg.getValue("filemanager") != "none") && (cfg.getValue("show filemanager item") != "false") )
                filemanager = format(MENU_ITEM[4..length], "File Manager", cfg.getValue("filemanager"));
            
            string runcommand = "";
            if ((cfg.getValue("run command") != "none") && (cfg.getValue("show run command item") != "false") )
                runcommand = format(MENU_ITEM[4..length], "Run Command", cfg.getValue("run command"));
                
            return format("%s%s%s%s", terminal, editor, filemanager, runcommand);
        }
        
        
        string getWindowsAndDesktopsMenu()
        {
            return  separator() ~ "    <menu id=\"client-list-combined-menu\" />\n";
        }
                
        
        string getCategoryMenu(string category)
        {
            string menu;
            CategoryInfo catInfo = CategoriesInfo[category];
            string[string] tmpInfo;
            foreach(key; keysList)
            {
                tmpInfo = infoList[key];
                foreach(cat; catInfo.categories)
                {
                    if (tmpInfo["Category"] == cat)
                    {
                        menu ~= menuitem(tmpInfo["Name"], tmpInfo["Exec"], tmpInfo["InTerminal"]);
                    }
                }
            }
            if (menu == "")
                return menu;
            return submenu(catInfo.label, catInfo.label, menu);
        }
        
        
        string getOpenBoxMenu()
        {
            string menu = "        <item label=\"Reload OpenBox Configuration\"><action name=\"Reconfigure\" /></item>\n";
    
            foreach(info; infoList)
            {
                if (info["Exec"].startsWith("obconf"))
                {
                    menu ~= menuitem(info["Name"], info["Exec"], info["InTerminal"]);
                    break;
                }
            }
            
            menu ~= format(MENU_ITEM, "Configure Autostarted Applications",
                                      format("%s %s", cfg.getValue("editor"), cfg.AUTOSTARTSH));
    
            menu ~= format(MENU_ITEM, "Edit rc.xml",
                                      format("%s %s", cfg.getValue("editor"), cfg.RCXML));
                
            menu ~= "    " ~ separator("obmenugen settings");

            menu ~= format(MENU_ITEM, "Edit obmenugen configuration",
                                      format("%s %s", cfg.getValue("editor"), cfg.OBMENUGEN_CFG));

            menu ~= format(MENU_ITEM, "Edit obmenugen exclutions",
                                      format("%s %s", cfg.getValue("editor"), cfg.OBMENUGEN_EXCLUSIONS_FILE));
    
            return separator() ~ submenu("OpenBox Settings", "OpenBox Settings", menu);
        }
        

        string getLockAndExitItems()
        {            
            string lock = "";
            if (cfg.getValue("lock command")!="none")
                lock = format(MENU_ITEM[4..length], "Lock Screen", cfg.getValue("lock command"));
 
            string exit;
            if (cfg.getValue("exit command")!="none")
                exit = format(MENU_ITEM, "Exit", cfg.getValue("exit command"));
            else
                exit = "    <item label=\"Exit\"><action name=\"Exit\" /></item>\n";
                
            return format("%s%s%s", separator(), lock, exit);
        }

        
        string getFooter(string menu_type)
        {
            string footer;
            
            if (menu_type == "staticXML")
                footer ~= "</menu>\n";
            
            if ( (menu_type == "staticXML") || (menu_type == "pipemenuXML") )
                footer ~= "</openbox_menu>\n";
            else
                footer = "</openbox_pipe_menu>\n";
            
            return footer;
        }

        
        
        string separator(string label = "")
        {
            if (label != "")
                label = format("label=\"%s\"", label);
            return format("    <separator %s/>\n", label);
        }
        
        
        string menuitem(string label, string exec_cmd, string in_terminal = "false")
        {
            if (in_terminal == "true")
                exec_cmd = format("%s -e %s", cfg.getValue("terminal"), exec_cmd);
            return format(MENU_ITEM, label, exec_cmd);
        }
        
        
        string submenu(string id, string label, string menuContent)
        {
            return format(SUBMENU, id, label, menuContent);
        }
        
        
        void initCategoriesInfo()
        {
            static CategoryInfo accesories = { categories:["utility"], label:"Accesories" };
            CategoriesInfo["accesories"] = accesories;
            static CategoryInfo graphics = { categories:["graphics"], label:"Graphics" };
            CategoriesInfo["graphics"] = graphics;
            static CategoryInfo audiovideo = { categories:["audiovideo", "audio", "video"], label:"Audio &amp; Video" };
            CategoriesInfo["audiovideo"] = audiovideo;
            static CategoryInfo office = { categories:["office"], label:"Office" };
            CategoriesInfo["office"] = office;
            static CategoryInfo games = { categories:["game"], label:"Games" };
            CategoriesInfo["games"] = games;
            static CategoryInfo network = { categories:["network"], label:"Network" };
            CategoriesInfo["network"] = network;
            static CategoryInfo devel = { categories:["development"], label:"Development" };
            CategoriesInfo["devel"] = devel;
            static CategoryInfo settings = { categories:["settings"], label:"Settings" };
            CategoriesInfo["settings"] = settings;
            static CategoryInfo system = { categories:["system"], label:"System Tools" };
            CategoriesInfo["system"] = system;
        }
}

